import { aws_ec2 as ec2, aws_s3 as s3 } from 'aws-cdk-lib';
import * as constructs from 'constructs';
/** Properties for flow logs **/
export interface FlowLogProps {
    /** the central s3 location for enterprise flow logs */
    readonly bucket: s3.Bucket;
    /** 1 minute resolution */
    readonly oneMinuteFlowLogs?: boolean;
    /** create in Account Athena Querys for flow logs*/
    readonly localAthenaQuerys?: boolean;
}
/** Propertys for Attaching to a Cloudwan Core Network */
export interface AttachToCloudWanProps {
    /** corenetworkName */
    readonly coreNetworkName: string;
    readonly segmentName: string;
    readonly attachmentSubnetGroup?: string | undefined;
}
/**
 * Propertys for Appliance Mode
 */
export declare enum ApplianceMode {
    /** enable Connecting VPC to TransitGateway in Appliance Mode */
    ENABLED = "enabled"
}
/**
 * Propertys to attach the Vpc To Transit Gateway
 */
export interface AttachToTransitGatewayProps {
    /** the TransitGateway to connect to */
    readonly transitGateway: ec2.CfnTransitGateway;
    /** Will this be connected in appliance mode ( used if you have Network Firewalls ) */
    readonly applicanceMode?: ApplianceMode | undefined;
    readonly attachmentSubnetGroup?: string | undefined;
}
/**
 * Propertys for Adding Routes in VPC.
 */
export interface AddRoutesProps {
    readonly cidr: string[];
    readonly subnetGroups: string[];
    readonly destination: Destination;
}
/**
 * The Destinations for Adding Routes
 */
export declare enum Destination {
    /** route to the cloudwan that the vpc is attached to */
    CLOUDWAN = "Cloudwan",
    /** route to the transitGateway that the vpc is attached to */
    TRANSITGATEWAY = "TransitGateway"
}
/** Propertys for an Enterprise VPC */
export interface EnterpriseVpcProps {
    readonly vpc: ec2.Vpc;
}
/**
 * Enteprise VPC's take the stock ec2.Vpc and provide numerous convience methods primarly related to
 * connecting to internal networks
 */
export declare class EnterpriseVpc extends constructs.Construct {
    /**
       * AttachmentId when the vpc is attached to a Cloudwan
       */
    cloudWanVpcAttachmentId: string | undefined;
    /**
       * AttachmentId when the vpc is attached to a transitGateway
       */
    transitGWAttachmentID: string | undefined;
    /**
       * The Id of the transitgateway that the VPC is attached to
       */
    transitGWID: string | undefined;
    /**
       * the Name of the cloudwan that the VPC is attached to
       */
    cloudWanName: string | undefined;
    /**
       * the Name of the Cloudwan segment that the vpc is attached to
       */
    cloudWanSegment: string | undefined;
    /**
       * the ec2.Vpc that is passed in as property
       */
    readonly vpc: ec2.Vpc;
    /**
       * Constructor
       * @param scope
       * @param id
       * @param props
       */
    constructor(scope: constructs.Construct, id: string, props: EnterpriseVpcProps);
    /**
       * Create Enterprise VPC Flow Logs (to central log account) and advanced diagnostics with Athena Querys
       * @param props
       */
    createFlowLog(props: FlowLogProps): void;
    /**
       * attachToCloudWan will attach a VPC to CloudWan, in a particular Segment.
       * @param props
       */
    attachToCloudWan(props: AttachToCloudWanProps): string;
    /**
       * Attach a vpc to a transit gateway, possibly in appliance mode
       * Its intended purpose is provide a
       * @param props
       */
    attachToTransitGateway(props: AttachToTransitGatewayProps): string;
    /**
       * Add routes to SubnetGroups ( by implication their routing tables )
       * @param props
       */
    addRoutes(props: AddRoutesProps): void;
}
